@@ -4,31 +4,31 @@ It does this via the Public Suffix List (PSL).
 
     >>> import tldextract
 
-    >>> tldextract.extract('http://forums.news.cnn.com/')
+    >>> tldextract.extract("http://forums.news.cnn.com/")
     ExtractResult(subdomain='forums.news', domain='cnn', suffix='com', is_private=False)
 
-    >>> tldextract.extract('http://forums.bbc.co.uk/') # United Kingdom
+    >>> tldextract.extract("http://forums.bbc.co.uk/")  # United Kingdom
     ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk', is_private=False)
 
-    >>> tldextract.extract('http://www.worldbank.org.kg/') # Kyrgyzstan
+    >>> tldextract.extract("http://www.worldbank.org.kg/")  # Kyrgyzstan
     ExtractResult(subdomain='www', domain='worldbank', suffix='org.kg', is_private=False)
 
 Note subdomain and suffix are _optional_. Not all URL-like inputs have a
 subdomain or a valid suffix.
 
-    >>> tldextract.extract('google.com')
+    >>> tldextract.extract("google.com")
     ExtractResult(subdomain='', domain='google', suffix='com', is_private=False)
 
-    >>> tldextract.extract('google.notavalidsuffix')
+    >>> tldextract.extract("google.notavalidsuffix")
     ExtractResult(subdomain='google', domain='notavalidsuffix', suffix='', is_private=False)
 
-    >>> tldextract.extract('http://127.0.0.1:8080/deployed/')
+    >>> tldextract.extract("http://127.0.0.1:8080/deployed/")
     ExtractResult(subdomain='', domain='127.0.0.1', suffix='', is_private=False)
 
 To rejoin the original hostname, if it was indeed a valid, registered hostname:
 
-    >>> ext = tldextract.extract('http://forums.bbc.co.uk')
-    >>> ext.registered_domain
+    >>> ext = tldextract.extract("http://forums.bbc.co.uk")
+    >>> ext.top_domain_under_public_suffix
     'bbc.co.uk'
     >>> ext.fqdn
     'forums.bbc.co.uk'
@@ -38,8 +38,9 @@ from __future__ import annotations
 
 import os
 import urllib.parse
+import warnings
 from collections.abc import Collection, Sequence
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from functools import wraps
 
 import idna
@@ -61,34 +62,59 @@ PUBLIC_SUFFIX_LIST_URLS = (
 class ExtractResult:
     """A URL's extracted subdomain, domain, and suffix.
 
-    Also contains metadata, like a flag that indicates if the URL has a private suffix.
+    These first 3 fields are what most users of this library will care about.
+    They are the split, non-overlapping hostname components of the input URL.
+    They can be used to rebuild the original URL's hostname.
+
+    Beyond the first 3 fields, the class contains metadata fields, like a flag
+    that indicates if the input URL's suffix is from a private domain.
     """
 
     subdomain: str
+    """All subdomains beneath the domain of the input URL, if it contained any such subdomains, or else the empty string."""
+
     domain: str
+    """The topmost domain of the input URL, if it contained a domain name, or else everything hostname-like in the input.
+
+    If the input URL didn't contain a real domain name, the `suffix` field will
+    be empty, and this field will catch values like an IP address, or
+    private network hostnames like "localhost".
+    """
+
     suffix: str
+    """The public suffix of the input URL, if it contained one, or else the empty string.
+
+    If `include_psl_private_domains` was set to `False`, this field is the same
+    as `registry_suffix`, i.e. a domain under which people can register
+    subdomains through a registrar. If `include_psl_private_domains` was set to
+    `True`, this field may be a PSL private domain, like "blogspot.com".
+    """
+
     is_private: bool
+    """Whether the input URL belongs in the Public Suffix List's private domains.
 
-    @property
-    def registered_domain(self) -> str:
-        """Joins the domain and suffix fields with a dot, if they're both set.
+    If `include_psl_private_domains` was set to `False`, this field is always
+    `False`.
+    """
 
-        >>> extract('http://forums.bbc.co.uk').registered_domain
-        'bbc.co.uk'
-        >>> extract('http://localhost:8080').registered_domain
-        ''
-        """
-        if self.suffix and self.domain:
-            return f"{self.domain}.{self.suffix}"
-        return ""
+    registry_suffix: str = field(repr=False)
+    """The registry suffix of the input URL, if it contained one, or else the empty string.
+
+    This field is a domain under which people can register subdomains through a
+    registar.
+
+    This field is unaffected by the `include_psl_private_domains` setting. If
+    `include_psl_private_domains` was set to `False`, this field is always the
+    same as `suffix`.
+    """
 
     @property
     def fqdn(self) -> str:
-        """Returns a Fully Qualified Domain Name, if there is a proper domain/suffix.
+        """The Fully Qualified Domain Name (FQDN), if there is a proper `domain` and `suffix`, or else the empty string.
 
-        >>> extract('http://forums.bbc.co.uk/path/to/file').fqdn
+        >>> extract("http://forums.bbc.co.uk/path/to/file").fqdn
         'forums.bbc.co.uk'
-        >>> extract('http://localhost:8080').fqdn
+        >>> extract("http://localhost:8080").fqdn
         ''
         """
         if self.suffix and (self.domain or self.is_private):
@@ -97,13 +123,13 @@ class ExtractResult:
 
     @property
     def ipv4(self) -> str:
-        """Returns the ipv4 if that is what the presented domain/url is.
+        """The IPv4 address, if that is what the input domain/URL was, or else the empty string.
 
-        >>> extract('http://127.0.0.1/path/to/file').ipv4
+        >>> extract("http://127.0.0.1/path/to/file").ipv4
         '127.0.0.1'
-        >>> extract('http://127.0.0.1.1/path/to/file').ipv4
+        >>> extract("http://127.0.0.1.1/path/to/file").ipv4
         ''
-        >>> extract('http://256.1.1.1').ipv4
+        >>> extract("http://256.1.1.1").ipv4
         ''
         """
         if (
@@ -116,13 +142,17 @@ class ExtractResult:
 
     @property
     def ipv6(self) -> str:
-        """Returns the ipv6 if that is what the presented domain/url is.
+        """The IPv6 address, if that is what the input domain/URL was, or else the empty string.
 
-        >>> extract('http://[aBcD:ef01:2345:6789:aBcD:ef01:127.0.0.1]/path/to/file').ipv6
+        >>> extract(
+        ...     "http://[aBcD:ef01:2345:6789:aBcD:ef01:127.0.0.1]/path/to/file"
+        ... ).ipv6
         'aBcD:ef01:2345:6789:aBcD:ef01:127.0.0.1'
-        >>> extract('http://[aBcD:ef01:2345:6789:aBcD:ef01:127.0.0.1.1]/path/to/file').ipv6
+        >>> extract(
+        ...     "http://[aBcD:ef01:2345:6789:aBcD:ef01:127.0.0.1.1]/path/to/file"
+        ... ).ipv6
         ''
-        >>> extract('http://[aBcD:ef01:2345:6789:aBcD:ef01:256.0.0.1]').ipv6
+        >>> extract("http://[aBcD:ef01:2345:6789:aBcD:ef01:256.0.0.1]").ipv6
         ''
         """
         min_num_ipv6_chars = 4
@@ -137,6 +167,123 @@ class ExtractResult:
                 return debracketed
         return ""
 
+    @property
+    def registered_domain(self) -> str:
+        """The `domain` and `suffix` fields joined with a dot, if they're both set, or else the empty string.
+
+        >>> extract("http://forums.bbc.co.uk").registered_domain
+        'bbc.co.uk'
+        >>> extract("http://localhost:8080").registered_domain
+        ''
+
+        .. deprecated:: 6.0.0
+           This property is deprecated and will be removed in the next major
+           version. Use `top_domain_under_public_suffix` instead, which has the
+           same behavior but a more accurate name.
+
+        This is an alias for the `top_domain_under_public_suffix` property.
+        `registered_domain` is so called because is roughly the domain the
+        owner paid to register with a registrar or, in the case of a private
+        domain, "registered" with the domain owner. If the input was not
+        something one could register, this property returns the empty string.
+
+        To distinguish the case of private domains, consider Blogspot, which is
+        in the PSL's private domains. If `include_psl_private_domains` was set
+        to `False`, the `registered_domain` property of a Blogspot URL
+        represents the domain the owner of Blogspot registered with a
+        registrar, i.e. Google registered "blogspot.com". If
+        `include_psl_private_domains=True`, the `registered_domain` property
+        represents the "blogspot.com" _subdomain_ the owner of a blog
+        "registered" with Blogspot.
+
+        >>> extract(
+        ...     "http://waiterrant.blogspot.com", include_psl_private_domains=False
+        ... ).registered_domain
+        'blogspot.com'
+        >>> extract(
+        ...     "http://waiterrant.blogspot.com", include_psl_private_domains=True
+        ... ).registered_domain
+        'waiterrant.blogspot.com'
+
+        To always get the same joined string, regardless of the
+        `include_psl_private_domains` setting, consider the
+        `top_domain_under_registry_suffix` property.
+        """
+        warnings.warn(
+            "The 'registered_domain' property is deprecated and will be removed in the next major version. "
+            "Use 'top_domain_under_public_suffix' instead, which has the same behavior but a more accurate name.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return self.top_domain_under_public_suffix
+
+    @property
+    def reverse_domain_name(self) -> str:
+        """The domain name in Reverse Domain Name Notation.
+
+        Joins extracted components of the input URL in reverse domain name
+        notation. The suffix is used as the leftmost component, followed by the
+        domain, then followed by the subdomain with its parts reversed.
+
+        Reverse Domain Name Notation is typically used to organize namespaces
+        for packages and plugins. Technically, a full reversal would reverse
+        the parts of the suffix, e.g. "co.uk" would become "uk.co", but this is
+        not done in practice when Reverse Domain Name Notation is called for.
+        So this property leaves the `suffix` part in its original order.
+
+        >>> extract("login.example.com").reverse_domain_name
+        'com.example.login'
+
+        >>> extract("login.example.co.uk").reverse_domain_name
+        'co.uk.example.login'
+        """
+        stack = [self.suffix, self.domain]
+        if self.subdomain:
+            stack.extend(reversed(self.subdomain.split(".")))
+        return ".".join(stack)
+
+    @property
+    def top_domain_under_registry_suffix(self) -> str:
+        """The rightmost domain label and `registry_suffix` joined with a dot, if such a domain is available and `registry_suffix` is set, or else the empty string.
+
+        The rightmost domain label might be in the `domain` field, or, if the
+        input URL's suffix is a PSL private domain, in the public suffix
+        `suffix` field.
+
+        If the input was not in the PSL's private domains, this property is
+        equivalent to `top_domain_under_public_suffix`.
+
+        >>> extract(
+        ...     "http://waiterrant.blogspot.com", include_psl_private_domains=False
+        ... ).top_domain_under_registry_suffix
+        'blogspot.com'
+        >>> extract(
+        ...     "http://waiterrant.blogspot.com", include_psl_private_domains=True
+        ... ).top_domain_under_registry_suffix
+        'blogspot.com'
+        >>> extract("http://localhost:8080").top_domain_under_registry_suffix
+        ''
+        """
+        top_domain_under_public_suffix = self.top_domain_under_public_suffix
+        if not top_domain_under_public_suffix or not self.is_private:
+            return top_domain_under_public_suffix
+
+        num_labels = self.registry_suffix.count(".") + 2
+        return ".".join(top_domain_under_public_suffix.split(".")[-num_labels:])
+
+    @property
+    def top_domain_under_public_suffix(self) -> str:
+        """The `domain` and `suffix` fields joined with a dot, if they're both set, or else the empty string.
+
+        >>> extract("http://forums.bbc.co.uk").top_domain_under_public_suffix
+        'bbc.co.uk'
+        >>> extract("http://localhost:8080").top_domain_under_public_suffix
+        ''
+        """
+        if self.suffix and self.domain:
+            return f"{self.domain}.{self.suffix}"
+        return ""
+
 
 class TLDExtract:
     """A callable for extracting, subdomain, domain, and suffix components from a URL."""
@@ -236,9 +383,9 @@ class TLDExtract:
         I.e. its effective TLD, gTLD, ccTLD, etc. components.
 
         >>> extractor = TLDExtract()
-        >>> extractor.extract_str('http://forums.news.cnn.com/')
+        >>> extractor.extract_str("http://forums.news.cnn.com/")
         ExtractResult(subdomain='forums.news', domain='cnn', suffix='com', is_private=False)
-        >>> extractor.extract_str('http://forums.bbc.co.uk/')
+        >>> extractor.extract_str("http://forums.bbc.co.uk/")
         ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk', is_private=False)
 
         Allows configuring the HTTP request via the optional `session`
@@ -271,9 +418,11 @@ class TLDExtract:
         name has already been parsed.
 
         >>> extractor = TLDExtract()
-        >>> extractor.extract_urllib(urllib.parse.urlsplit('http://forums.news.cnn.com/'))
+        >>> extractor.extract_urllib(
+        ...     urllib.parse.urlsplit("http://forums.news.cnn.com/")
+        ... )
         ExtractResult(subdomain='forums.news', domain='cnn', suffix='com', is_private=False)
-        >>> extractor.extract_urllib(urllib.parse.urlsplit('http://forums.bbc.co.uk/'))
+        >>> extractor.extract_urllib(urllib.parse.urlsplit("http://forums.bbc.co.uk/"))
         ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk', is_private=False)
         """
         return self._extract_netloc(
@@ -297,26 +446,60 @@ class TLDExtract:
             len(netloc_with_ascii_dots) >= min_num_ipv6_chars
             and netloc_with_ascii_dots[0] == "["
             and netloc_with_ascii_dots[-1] == "]"
+            and looks_like_ipv6(netloc_with_ascii_dots[1:-1])
         ):
-            if looks_like_ipv6(netloc_with_ascii_dots[1:-1]):
-                return ExtractResult("", netloc_with_ascii_dots, "", is_private=False)
+            return ExtractResult(
+                "", netloc_with_ascii_dots, "", is_private=False, registry_suffix=""
+            )
 
         labels = netloc_with_ascii_dots.split(".")
 
-        suffix_index, is_private = self._get_tld_extractor(
-            session=session
-        ).suffix_index(labels, include_psl_private_domains=include_psl_private_domains)
+        maybe_indexes = self._get_tld_extractor(session).suffix_index(
+            labels, include_psl_private_domains=include_psl_private_domains
+        )
 
         num_ipv4_labels = 4
-        if suffix_index == len(labels) == num_ipv4_labels and looks_like_ip(
-            netloc_with_ascii_dots
+        if (
+            not maybe_indexes
+            and len(labels) == num_ipv4_labels
+            and looks_like_ip(netloc_with_ascii_dots)
         ):
-            return ExtractResult("", netloc_with_ascii_dots, "", is_private)
+            return ExtractResult(
+                "", netloc_with_ascii_dots, "", is_private=False, registry_suffix=""
+            )
+        elif not maybe_indexes:
+            return ExtractResult(
+                subdomain=".".join(labels[:-1]),
+                domain=labels[-1],
+                suffix="",
+                is_private=False,
+                registry_suffix="",
+            )
 
-        suffix = ".".join(labels[suffix_index:]) if suffix_index != len(labels) else ""
-        subdomain = ".".join(labels[: suffix_index - 1]) if suffix_index >= 2 else ""
-        domain = labels[suffix_index - 1] if suffix_index else ""
-        return ExtractResult(subdomain, domain, suffix, is_private)
+        (
+            (public_suffix_index, public_suffix_node),
+            (registry_suffix_index, registry_suffix_node),
+        ) = maybe_indexes
+
+        subdomain = (
+            ".".join(labels[: public_suffix_index - 1])
+            if public_suffix_index >= 2
+            else ""
+        )
+        domain = labels[public_suffix_index - 1] if public_suffix_index > 0 else ""
+        public_suffix = ".".join(labels[public_suffix_index:])
+        registry_suffix = (
+            ".".join(labels[registry_suffix_index:])
+            if public_suffix_node.is_private
+            else public_suffix
+        )
+        return ExtractResult(
+            subdomain=subdomain,
+            domain=domain,
+            suffix=public_suffix,
+            is_private=public_suffix_node.is_private,
+            registry_suffix=registry_suffix,
+        )
 
     def update(
         self, fetch_now: bool = False, session: requests.Session | None = None
@@ -329,9 +512,9 @@ class TLDExtract:
 
     @property
     def tlds(self, session: requests.Session | None = None) -> list[str]:
-        """Returns the list of tld's used by default.
+        """The list of TLDs used by default.
 
-        This will vary based on `include_psl_private_domains` and `extra_suffixes`
+        This will vary based on `include_psl_private_domains` and `extra_suffixes`.
         """
         return list(self._get_tld_extractor(session=session).tlds())
 
@@ -473,40 +656,49 @@ class _PublicSuffixListTLDExtractor:
 
     def suffix_index(
         self, spl: list[str], include_psl_private_domains: bool | None = None
-    ) -> tuple[int, bool]:
-        """Return the index of the first suffix label, and whether it is private.
+    ) -> tuple[tuple[int, Trie], tuple[int, Trie]] | None:
+        """Return the index of the first public suffix label, the index of the first registry suffix label, and their corresponding trie nodes.
 
-        Returns len(spl) if no suffix is found.
+        Returns `None` if no suffix is found.
         """
         if include_psl_private_domains is None:
             include_psl_private_domains = self.include_psl_private_domains
 
-        node = (
+        node = reg_node = (
             self.tlds_incl_private_trie
             if include_psl_private_domains
             else self.tlds_excl_private_trie
         )
-        i = len(spl)
-        j = i
+        suffix_idx = reg_idx = label_idx = len(spl)
         for label in reversed(spl):
             decoded_label = _decode_punycode(label)
             if decoded_label in node.matches:
-                j -= 1
+                label_idx -= 1
                 node = node.matches[decoded_label]
                 if node.end:
-                    i = j
+                    suffix_idx = label_idx
+                    if not node.is_private:
+                        reg_node = node
+                        reg_idx = label_idx
                 continue
 
             is_wildcard = "*" in node.matches
             if is_wildcard:
                 is_wildcard_exception = "!" + decoded_label in node.matches
-                if is_wildcard_exception:
-                    return j, node.matches["*"].is_private
-                return j - 1, node.matches["*"].is_private
+                return (
+                    label_idx if is_wildcard_exception else label_idx - 1,
+                    node.matches["*"],
+                ), (
+                    reg_idx,
+                    reg_node,
+                )
 
             break
 
-        return i, node.is_private
+        if suffix_idx == len(spl):
+            return None
+
+        return ((suffix_idx, node), (reg_idx, reg_node))
 
 
 def _decode_punycode(label: str) -> str:
